% INTERSECT    Geometric intersection of a polygon and halfplane
%   ***** Inputs *****
%   polygon - a 2 x n array defining the vertices of a polygon in 2D
%   b0 - a point on the dividing line
%   b1 - a second point on the dividing line.  The vector b1-b0 defines the
%       intersecting halfplane.  The intersection of the ray with the
%       polygon would define a new positvely oriented edge.  If the polygon
%       is to the left of the ray, the intersection is the entire polygon.
%       If the polygon is to the right of the ray, the intersection is the
%       null set.
%
%   ***** Outputs *****
%   vertices - the vertices of the intersection
function vertices = intersect(polygon, b0, b1)
b0 = b0(:);
b1 = b1(:);
n = size(polygon, 2);
if (n == 0)
    vertices = [];
    return;
end

% Determine which segments the halfplane intersects
intersections = zeros(n, 1);
newPoints = zeros(2, 1, n);
for i = 1:n
    a0 = polygon(:, i);
    if (i == n)
        a1 = polygon(:, 1);
    else
        a1 = polygon(:, i+1);
    end
    [bool, pt] = isIntersect(a0, a1, b0, b1);
    if (bool)
        intersections(i) = 1;
        newPoints(:,:,i) = pt;
    end
end

% Now figure out which half of the polygon to cut.
if (max(intersections) == 1)    % if the halfplane cuts thru the polygon
    intersectIndex = [0, 0]';
    ctr = 1;
    for i = 1:n
        if (intersections(i) == 1)
            intersectIndex(ctr) = i;
            ctr = ctr + 1;
        end
    end
    % Check positive orientation of the intersected points
    p1 = newPoints(:,:,intersectIndex(1));
    p2 = newPoints(:,:,intersectIndex(2));
    vec = p2 - p1;
    if ((vec' * (b1 - b0)) < 0) % in the wrong direction
        p1 = p2;
        p2 = newPoints(:,:,intersectIndex(1));
        temp = intersectIndex(1);
        intersectIndex(1) = intersectIndex(2);
        intersectIndex(2) = temp;
    end
    % Begin definition of new polygon starting with the cutting segment
    vertices = [p1, p2];
    polygon = circshift(polygon, [0, -intersectIndex(2)]);
    if (intersectIndex(2) > intersectIndex(1))
        cutSize = intersectIndex(2) - intersectIndex(1);
    else
        cutSize = intersectIndex(2) - intersectIndex(1) + n;
    end
    vertices(:, end+1:end+n-cutSize) = polygon(:, 1:n-cutSize);
else
    % no intersection with polygon, but depending on orientation, the 
    % halfplane will either include or exclude the polygon.  Check using
    % cross-product.
    v1 = b1 - b0;
    for i = 1:n
        p = polygon(:,i) - b0;
        % c = cross(v1, p);
        c = v1(1)*p(2) - v1(2)*p(1);
        if (c < 0)
            vertices = [];
            return
        end
    end
    vertices = polygon;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% helper function to detect intersection of a segment u0->u1 with a line
% v0->v1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [bool, pt] = isIntersect(u0, u1, v0, v1)
a1 = u1(1) - u0(1);
a2 = u1(2) - u0(2);
b1 = v1(1) - v0(1);
b2 = v1(2) - v0(2);
c1 = v0(1) - u0(1);
c2 = v0(2) - u0(2);
D = a2*b1 - a1*b2;
if (D == 0)     % parallel lines
    bool = 0;
    pt = NaN;
    return;
end
t = -b2*c1/D + b1*c2/D;
% s = -a2*c1/D + a1*c2/D
% deal with numerical precision
if (t < -eps(10) || t > 1 + eps(10))
    bool = 0;
    pt = 0;
elseif (0<=t && t<=1)
    bool = 1;
    pt = u0 + (u1 - u0)*t;
elseif (abs(t) <= eps(10))
    bool = 1;
    pt = u0;
elseif (abs(1-t) <= eps(10))
    bool = 1;
    pt = u1;
end